from casadi import *
import numpy
from matplotlib import pylab as plt

scale = 0.02
offset = 0.7

DEBUG=False
N = 40
mi = 40.0/N
Di = 70.0*N
g0 = 9.81

p = SX.sym('p')
yz = SX.sym('yz',2*N)
y = yz[0::2]
z = yz[1::2]

# f
f = 0
for k in range(N-1):
    f += 0.5*Di*( (y[k] - y[k+1])**2 + (z[k] - z[k+1])**2 )
for k in range(N):
    f += mi*g0*z[k]

# box constraints
lbx = -inf*numpy.ones((2*N))
ubx =  inf*numpy.ones((2*N))
lbx[ 0] = ubx[ 0] = -2
lbx[ 1] = ubx[ 1] =  1
lbx[-2] = ubx[-2] =  2
lbx[-1] = ubx[-1] =  1

if DEBUG:
    print "lower bound:"
    print lbx
    print "upper bound:"
    print ubx

# introduce slanted ground constraints
g = veccat([z[k] - offset - p*scale*y[k]**2 for k in range(N)])

if DEBUG:
    print "g:"
    print g

nlp = SXFunction( nlpIn(x=yz, p=p), nlpOut(f=f, g=g) )
solver = NlpSolver('ipopt', nlp)
solver.init()

solver.setInput(lbx,'lbx')
solver.setInput(ubx,'ubx')
solver.setInput(numpy.zeros((N,1)), 'lbg')
solver.setInput(inf*numpy.ones((N,1)), 'ubg')
solver.setInput(1,'p')
solver.evaluate()
xopt0 = solver.getOutput('x')
Y0 = xopt0[0::2]
Z0 = xopt0[1::2]

solver.setInput(-1,'p')
solver.evaluate()
xopt1 = solver.getOutput('x')
Y1 = xopt1[0::2]
Z1 = xopt1[1::2]


plt.plot(Y0,Z0,'bo-')
plt.plot(Y1,Z1,'ro-')

ys = numpy.linspace(-2,2,100)
plt.plot(ys,offset + scale*ys**2,'b')
plt.plot(ys,offset - scale*ys**2,'r')


plt.xlabel('y [m]')
plt.ylabel('z [m]')
plt.title('hanging chain QP')
plt.grid(True)
plt.legend(['z >= 0.7 + 0.02*y**2', 'z >= 0.7 - 0.02*y**2'],loc=9)
plt.show()
